#!/bin/bash
########################################################################
# build_pygccxml.sh
#
# Build pygccxml and package for UPS.
#
# Adapted from build.sh.template provided by ssibuildshims v2_15_00.
########################################################################

####################################
# Useful variables.
prog=${BASH_SOURCE[0]##*/}

##################
# Local (dumb) ssi_die() function for use until we source ssi_functions.
ssi_die() {
  printf "FATAL ERROR: $*\n" 1>&2; exit 1
}
##################

##################
# Basic usage function for use by ensure_buildshims until we have enough
# information to provide something more specific. Do NOT customize!
build_script_usage() {
    cat <<EOF
USAGE: $prog <ssi-msg-config-options> [--] <product-dir> <qualifier-args> <maketar>
       $prog -h|--help [<product-dir>]

  Build $package and package for UPS.

OPTIONS

  -h|--help

    Help. Supply <product-dir> for specific, detailed information on
    invoking $prog.

  <ssi-msg-config-options>

    Supply <product-dir> for more information.

ARGUMENTS

  <product-dir>

    The top-level UPS directory in which the source code distribution
    was unpacked. When specified with -h|--help, this enables us to
    provide more specific help about invoking $prog.

  <qualifier-args>
  <maketar>

    Supply <product-dir> for more information.

EOF
}
##################

##################
# Make sure we have UPS and ssibuildshims set up.
ensure_ssibuildshims() {
  if ! (( $# )) || [ "${@: -1}" = "-h" ] || [ "${@: -1}" = "--help" ]; then
    build_script_usage
    exit 1
  fi
  # Find <product-dir> from our arguments:
  local arg
  for arg in "$@"; do
    [[ "$arg" ==  -* ]] && continue;
    product_dir="$arg"; break
  done
  if [ -z "$product_dir" ]; then
    # Bad usage.
    ssi_die "Bad <product-dir> $product_dir\n$(build_script_usage)"
    exit 2
  elif ! { [ -d "$product_dir/.upsfiles" ] && [ -w "$product_dir" ]; }; then
    # Something wrong with <product-dir>.
    ssi_die "<product-dir> not a valid, writable unified UPS directory: " \
      "$product_dir\n$(build_script_usage)"
  fi
  # Promising...
  local abs_product_dir="$(cd "$product_dir" 2>/dev/null 2>&1 && pwd -P)"
  (( $? )) && ssi_die "unable to cd to <product-dir>: $product_dir\n" \
    "$(build_script_usage)"
  # Find our own bootstraps and yank 'em.
  local ssibuildshims_topdir="$product_dir/ssibuildshims/$ssibuildshims_version"
  [ -f "$ssibuildshims_topdir/bin/ssi_functions" ] && \
    source "$ssibuildshims_topdir/bin/ssi_functions" >/dev/null 2>&1 && \
    ensure_ups "$product_dir" || ssi_die "unable to setup UPS"
    setup ssibuildshims $ssibuildshims_version -z "$product_dir" || \
    ssi_die "unable to find ssibuildshims $ssibuildshims_version in
$product_dir and set it up with UPS."
  # Absolute is safer.
  absolute_path -- "$product_dir" product_dir
}
##################

####################################
# Useful variables. *C*
trace=maybe_xtrace # Could set to xtrace instead for unconditional.
thisdir=$(cd "${BASH_SOURCE%/*}" && pwd -P)
(( ssi_trace = 1 )) # Trace certain commands by default.
####################################

########################################################################
# Customize-able functions.
########################################################################

##################
# Ascertain whether we built and installed correctly. Add more /
# different calls to verify_in_filesystem as appropriate. Arguments
# (e.g. -q) are passed through to verify_in_filesystem.
is_built() {
  local func=$FUNCNAME
  local delegate="${func}_${pqual:0:2}"
  [ "$(type -t "$delegate")" == function ] || \
    ssi_internal "$func() unable to delegate to $delegate"
  $trace "$delegate" "$@"
}

is_built_p2() { # *C*
  $trace verify_in_filesystem "$@" -d "$site_packages_dir" && \
    $trace verify_in_filesystem "$@" -sf "$egg_file" && \
    { ( cd ${WORKSPACE:-${TMPDIR:-/var/tmp}} && \
          "$python" -c "import pygccxml" ) || \
        ssi_die "defective installation: $egg_file is present but cannot be loaded"
    }
}

is_built_p3() { # *C*
  $trace verify_in_filesystem "$@" \
         -d "$pkgdir/"lib*"/python$pythonMdotm"*"/site-packages/$pkgtarname.dist-info" && \
    { ( cd ${WORKSPACE:-${TMPDIR:-/var/tmp}} && \
          "$python" -c "import pygccxml" ) || \
        ssi_die "defective installation: unable to load $package $pkgdotver"
    }
}
##################

##################
# If a distribution archive is wanted and the installation is successful
# according to is_built(), then make it. Any arguments (e.g. -q) are
# passed through both to is_built() and make_basic_distribution().
#
# If make_basic_distribution() finds that the product is declared
# correctly and attempts to make the distribution, then it will cause us
# to exit with the status code of that operation. If not, it returns an
# error code.
#
# See make_basic_distribution -h for useful options to add to
# mbd_opts(). *C*
maybe_make_distribution() {
  if [ "$maketar" = "tar" ]; then
    # Customize mbd_opts. *C*
    local mbd_opts=(-d lib/$PYTHON_LIBDIR)
    $trace is_built "$@" && \
      $trace make_basic_distribution "$@" "${mbd_opts[@]}"
  fi
}
##################

########################################################################
# Package-specific functions.
########################################################################

install_for() {
  local func=$FUNCNAME
  local delegate="${func}_${pqual:0:2}"
  [ "$(type -t "$delegate")" == function ] || \
    ssi_internal "$func() unable to delegate to $delegate"
  $trace "$delegate" "$@"
}

install_for_p2() {
  ssi_info "installing for Python $pythonMdotm with setup.py"
  $trace mkdir -p "$site_packages_dir" || \
  ssi_die "Failed to create the site_packages directory $site_packages_dir"

  $trace "$python" setup.py install --prefix "$pkgdir" || \
    ssi_die "python setup.py install failed"
}

install_for_p3() {
  ssi_info "installing for Python $PythonMdotm with pip"

  # Somewhere for temporary installations of pip:
  pip_install_top="${WORKSPACE:-${TMPDIR:-/var/tmp/$USER.$$}}/PythonTmp"
#  trap "rm -rf \"$pip_install_top\" 2>/dev/null" EXIT
  local pip_install_tree="$pip_install_top/python$pythonMdotm"
  $trace mkdir -p "$pip_install_tree" || \
    ssi_die "unable to prepare temporary Python area for pip"
  ssi_info "installing latest pip in a temporary area"
  $trace env PATH=$(dropit -sfe "$pip_install_tree/bin") \
         PYTHONPATH=$(dropit -sfe -p "$PYTHONPATH" \
                             "$pip_install_tree/lib/python$pythonMdotm/site-packages") \
         pip3 install -I --no-cache-dir --disable-pip-version-check \
         --upgrade-strategy only-if-needed \
         --prefix "$pip_install_tree" -U pip || \
    ssi_die "unable to ensure a recent version of pip"

  local install_opts=(
    -I
    --disable-pip-version-check
    --no-cache-dir
    --prefix "$pkgdir"
    --upgrade-strategy only-if-needed
    --use-pep517
  )

  ssi_info "installing $package $pkgdotver with pip3"
  $trace env PATH=$(dropit -sfe "$pip_install_tree/bin") \
         PYTHONPATH=$(dropit -sfe -p "$PYTHONPATH" \
                             "$pkgdir/"{lib,lib64}"/python$pythonMdotm/site-packages" \
                             "$pip_install_tree/lib/python$pythonMdotm/site-packages") \
         pip3 install ${install_opts[*]:+"${install_opts[@]}"} . || \
    ssi_die "pip3 install of $package $pkgdotver FAILED"

  ssi_info "finding Python files and scripts for postprocessing..."
  IFS=$'\n' read -r -d '' -a pfiles < \
     <(find "$pkgdir" -type f \( \( -name '*.py' -print \) -o \
            \( -executable -exec file -e compress -e elf -e tar --mime-type \{\} \; \) \) | \
         sed -Ene '/:/ ! p; s&:[[:space:]]+text/.*$&&p')

  ssi_info "postprocessing Python files and scripts for relocatability..."

  postprocess_python_files "${pfiles[@]}"|| \
    ssi_die "postprocessing of python files for $package $pkgver"

  ssi_info "recompiling postprocessed Python source for $package $pkgdotver with Python $pythonMdotm"

  $trace "$python" -m compileall -j ${ncores:-1} "$pkgdir/bin" \
         "$pkgdir/"{lib,lib64}"/python$pythonMdotm/site-packages" || \
    ssi_die "unable to recompile postprocessed Python source for $package $pkgdotver with Python $pythonMdotm"
}

########################################################################
# Main.
########################################################################

####################################
# Package name and version information. *C*
. $thisdir/version_info.sh || \
  ssi_die "unable to load version info from $thisdir/version_info"
####################################

####################################
# Handle options and arguments.

# Set up ssibuildshims and UPS / product_dir as early as possible.
ensure_ssibuildshims "$@"

##################
# Now parse options and arguments, setting basequal, extraqual, maketar,
# etc.
#
# Use verify_quals() to parse command line options, verify that the user
# has provided qualifiers supported by this build script, and set
# expected variables such as basequal, extraqual, cqual, etc.
#
# Customize the extra usage text as appropriate. Note that DEBUG_PATCHES
# and VERBOSE_PATCHES are honored by prepare_source() and
# patch_source(). DEBUG_(CONFIG|BUILD|TEST|INSTALL) should be accounted
# for (or not) by the relevant command in this script.
#
# See verify_quals -h for options and examples.
verify_quals \
  --pqual={p2715a,p372,p382,p383b,p392} \
  --parse --usage-file=- -- "$@" <<EOF || exit # *C*
ENVIRONMENT

  DEBUG_(CONFIG|BUILD|TEST|INSTALL)

    Verbose output from the corresponding step.

  DEBUG_PATCHES
  VERBOSE_PATCHES

    Command tracing and increased verbosity for source unpacking and patching
    operations. DEBUG_PATCHES will cause exit after patches have been applied.
EOF
##################

##################
# Define globals: pkgdir, sourcedir, blddir, fullqual, etc. See
# define_basics -h for steering options.
$trace define_basics IN_SOURCE NO_BASE_QUAL NO_FQ_DIR NO_TYPE NULL \
       TRACK_PROGRESS || exit
##################

####################################
# Declare ourselves privately, and then set ourselves up to allow setup
# of dependencies. *C*
#
# Comment or delete if not required:
$trace fake_declare_product "${ssi_args[@]}" -x -- || exit
####################################

####################################
# Build-only dependencies.
####################################

# Print active products.
echo
ups active
echo

# Find Python in time for use by maybe_make_distribution.
python_vars
site_packages_dir="$pkgdir/lib/$PYTHON_LIBDIR/site-packages/"
egg_file="$site_packages_dir/$pkgtarname-py$pythonMdotm.egg"

# Possible short-circuit if we're already built and declared.
$trace maybe_make_distribution --silent

ssi_info "building $package for $OS-$plat-$qualdir (flavor $pkgflvr)"

# Ensure directories: pkgdir, sourcedir, blddir.
$trace init_pkg_dirs || exit

####################################
# Prepare source for building. *C*

$trace cd "$sourcedir" \
  || ssi_die "unable to cd to source directory $sourcedir"

# Source tarballs and patches. See prepare_source -h / patch_source -h
# for options.
$trace prepare_source "${ssi_args[@]}" -P "$patchdir" \
  "$tardir/$pkgtarfile" \
  || exit

# Bail early if DEBUG_PATCHES is set.
[ -n "$DEBUG_PATCHES" ] && exit 1
####################################

####################################
# Prepare to build. *C*
#
# Set configure / compile flags, environment variables, etc.

##################
# Configure parallelism. *C*
ncores=$(ncores)
####################################

####################################
# Configure, build, and install.
#
# Use $blddir/$pkgtarname for in-source builds, $blddir otherwise.

$trace cd "$blddir/$pkgtarname" || \
  ssi_die "unable to cd to build directory $blddir"

##################
# Configure.
# Build.
# Test.
# Install.
$trace install_for || \
  ssi_die "unable to install via install_for()"

ssi_info "finished building $package $pkgver"

####################################
# Pre-declaration processing. *C*

# Smoke test installation.
$trace "$python" -c "import pygccxml" || \
  ssi_die "Python failed to find pygccxml package"

##################
# Declare and set up the installed package.
$trace declare_product -x -- || exit
##################

####################################
# Post-declaration processing. *C*
####################################

#################
# Verify installation and make distribution archive.
$trace is_built && $trace ssi_build_completed && \
  ssi_info "$package is installed at $pkgdir" && \
  $trace maybe_make_distribution || \
    ssi_build_failed
########################################################################
# Nothing beyond this point.
########################################################################
